package org.playframework.playtools.preferences;


import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.playframework.playtools.util.PlayToolsUtil;

public class LocationEditor extends FieldEditor {

	private Table  table = null;
    private Button addBtn = null;
    private Button editBtn = null;
    private Button removeBtn = null;
    private Button upBtn = null;
    private Button downBtn = null;

    private final int TABLE_MAXROWS = 6;
    private final int BUTTON_WIDTH = 100;
    private final int UP = -1;
    private final int DOWN = 1;

	protected LocationEditor() {
	}

	protected LocationEditor(String name, String text, Composite parent) {
		init(name, text);
		createControl(parent);
	}

	@Override
	protected void adjustForNumColumns(int numColumns) {
	}

	@Override
	protected void doFillIntoGrid(Composite parent, int numColumns) {
        Control control = getLabelControl(parent);
        GridData gridControl = new GridData();
        gridControl.horizontalSpan = numColumns;
        control.setLayoutData(gridControl);

		Composite comp = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		comp.setLayout(layout);
		comp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		comp.setVisible(true);

		createList(comp);
        createListEditButtons(comp);

        listSelectionChange();
	}

	@Override
	protected void doLoad() {
        if (table != null) {
            String locations = getPreferenceStore().getString(getPreferenceName());
            List<String[]> listArray = PlayToolsUtil.paserLocation(locations);
            for (String[] items : listArray) {
                TableItem tableItem = new TableItem(table, 0);
                tableItem.setText(items);
            }
        }
	}

	@Override
	protected void doLoadDefault() {
	}

	@Override
	protected void doStore() {
        List<String[]> list = new ArrayList<String[]>();
        for (TableItem tableItem : table.getItems()) {
        	list.add(new String[] {tableItem.getText(0).trim(), tableItem.getText(1).trim(), tableItem.getText(2).trim()});
        }
        getPreferenceStore().setValue(getPreferenceName(), PlayToolsUtil.wordsToString(list));
	}

	@Override
	public int getNumberOfControls() {
		return 2;
	}

	private SelectionAdapter listSelectionListener = new SelectionAdapter() {
		public void widgetSelected(SelectionEvent e) {
			listSelectionChange();
		}
	};

    private void createList(Composite parent) {

	    table = new Table(parent, SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER);

	    GridData tableGrid = new GridData();
		tableGrid.horizontalAlignment = GridData.FILL;
		tableGrid.verticalAlignment = GridData.FILL;
		tableGrid.grabExcessHorizontalSpace = true;
		tableGrid.grabExcessVerticalSpace = true;
		tableGrid.verticalSpan = 5;
		tableGrid.widthHint = 550;
		tableGrid.heightHint = table.getItemHeight() * TABLE_MAXROWS;

		table.setLayoutData(tableGrid);
		table.setLinesVisible(true);
		table.setHeaderVisible(true);

		TableColumn col1 = new TableColumn(table, SWT.LEFT);
		col1.setText("Version");
		col1.setWidth(150);

		TableColumn col2 = new TableColumn(table, SWT.LEFT);
		col2.setText("Location");
		col2.setWidth(250);

		TableColumn col3 = new TableColumn(table, SWT.LEFT);
		col3.setText("Server port");
		col3.setWidth(150);

		table.addSelectionListener(listSelectionListener);

    }

    private void listSelectionChange() {
        int index = table.getSelectionIndex();
        int count = table.getItemCount();

        addBtn.setEnabled(count < TABLE_MAXROWS);
        editBtn.setEnabled(index >= 0);
        removeBtn.setEnabled(index >= 0);
        upBtn.setEnabled(count > 1 && index > 0);
        downBtn.setEnabled(count > 1 && index >= 0 && index < count - 1);

    }

    private SelectionAdapter addBtnSelectionListener = new SelectionAdapter() {
    	public void widgetSelected(SelectionEvent e) {
    		addLocation();
    	}
    };

    private SelectionAdapter editBtnSelectionListener = new SelectionAdapter() {
    	public void widgetSelected(SelectionEvent e) {
    		editLocation();
    	}
    };

    private SelectionAdapter removeBtnSelectionListener = new SelectionAdapter() {
    	public void widgetSelected(SelectionEvent e) {
    		removeLocation();
    	}
    };

    private SelectionAdapter upBtnSelectionListener = new SelectionAdapter() {
    	public void widgetSelected(SelectionEvent e) {
    		swapLocation(UP);
    	}
    };

    private SelectionAdapter downBtnSelectionListener = new SelectionAdapter() {
    	public void widgetSelected(SelectionEvent e) {
    		swapLocation(DOWN);
    	}
    };

    private void createListEditButtons(Composite parent) {
    	addBtn = createButton(parent, "Add", addBtnSelectionListener);
    	editBtn = createButton(parent, "Edit", editBtnSelectionListener);
    	removeBtn = createButton(parent, "Remove", removeBtnSelectionListener);
    	upBtn = createButton(parent, "Up", upBtnSelectionListener);
    	downBtn = createButton(parent, "Down", downBtnSelectionListener);
    }

    private Button createButton(Composite parent, String label, SelectionAdapter selectionAdapter) {
        Button button = new Button(parent, SWT.PUSH);
        button.setText(label);
        GridData buttonData = new GridData();
        buttonData.widthHint = BUTTON_WIDTH;
        button.setLayoutData(buttonData);
        button.addSelectionListener(selectionAdapter);
        return button;
    }

    protected void addLocation() {
    	LocationDialog d = new LocationDialog(addBtn.getShell());
    	int ret = d.open();

    	if (ret == IDialogConstants.OK_ID) {
            TableItem tableItem = null;
            String version = d.getPlayVersion();

    		int index = getVersionIndex(version);

    		if (index != -1) {
    			String msg;
    			msg = "Play Framework Version:" + version + " Override to Directory ?";

    			MessageDialog dialog = new MessageDialog(addBtn.getShell(),
    												"Play Framework Install Directory",
    												null,
    												msg,
    												MessageDialog.QUESTION,
    												new String[]{"Yes", "No"}, 1);
    			if (dialog.open() != MessageDialog.OK) {
    				return;
    			}
    			tableItem = table.getItem(index);
    		} else {
	            index = table.getSelectionIndex();
	            if (index >= 0) {
	                tableItem = new TableItem(table, 0, index+1);
	            } else {
	                tableItem = new TableItem(table, 0);
	            }
    		}
            String[] input = new String[] {d.getPlayVersion(), d.getPlayDirectory(), d.getServerPort()};

            tableItem.setText(input);

            listSelectionChange();
    	}
    }

    private void editLocation() {
        int index = table.getSelectionIndex();
        if (index < 0) {
        	return;
        }
        TableItem tableItem = table.getItem(index);

        LocationDialog d = new LocationDialog(addBtn.getShell());
        d.setPlayVersion(tableItem.getText(0));
        d.setPlayDirectory(tableItem.getText(1));
        d.setServerPort(tableItem.getText(2));

    	int ret = d.open();

    	if (ret == IDialogConstants.OK_ID) {
            String[] input = new String[] {d.getPlayVersion(), d.getPlayDirectory(), d.getServerPort()};

            tableItem.setText(input);

            listSelectionChange();
    	}
    }

    private int getVersionIndex(String version) {
    	int count = table.getItemCount();
    	for (int i = 0; i < count; i++) {
            TableItem targetItem = table.getItem(i);
    		if (targetItem.getText(0).equals(version)) {
    			return i;
    		}
    	}
    	return -1;
    }

    private void swapLocation(int up) {
        setPresentsDefaultValue(false);
        int index = table.getSelectionIndex();
        int target = index + up;

        if (index >= 0) {
            TableItem targetItem = table.getItem(target);
            TableItem sourceItem = table.getItem(index);

            String[] targetContents = new String[]{targetItem.getText(0), targetItem.getText(1), targetItem.getText(2)};
            String[] sourceContents = new String[]{sourceItem.getText(0), sourceItem.getText(1), sourceItem.getText(2)};

            targetItem.setText(sourceContents);
            sourceItem.setText(targetContents);

            table.setSelection(target);
        }
        listSelectionChange();
    }

    private void removeLocation() {
        setPresentsDefaultValue(false);
        int index = table.getSelectionIndex();
        if (index >= 0) {
            table.remove(index);
            listSelectionChange();
        }
    }
}
