package org.playframework.playtools.wizards;

import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.core.runtime.*;
import java.lang.reflect.InvocationTargetException;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.ui.*;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.playframework.playtools.util.config.PlayConfigManager;

/**
 * This is a sample new wizard. Its role is to create a new file
 * resource in the provided container. If the container resource
 * (a folder or a project) is selected in the workspace
 * when the wizard is opened, it will accept it as the target
 * container. The wizard creates one file with the extension
 * "mpe". If a sample multi-page editor (also available
 * as a template) is registered for the same extension, it will
 * be able to open it.
 */

public class PlayNewProjectWizard extends Wizard implements INewWizard {

	protected PlayNewProjectWizardPage wizardPage;

	protected ISelection selection;

	protected IWorkbench workbench;

	private IProject projectHandle;

	/**
	 * Constructor for SampleNewWizard.
	 */
	public PlayNewProjectWizard() {
		super();
		setNeedsProgressMonitor(true);
	}

	/**
	 * We will accept the selection in the workbench to see if
	 * we can initialize from it.
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
        this.workbench = workbench;
	}

	/**
	 * Adding the page to the wizard.
	 */

	public void addPages() {
		wizardPage = new PlayNewProjectWizardPage(selection);
		addPage(wizardPage);
	}

	/**
	 * This method is called when 'Finish' button is pressed in
	 * the wizard. We will create an operation and run it
	 * using wizard as execution context.
	 */
	public boolean performFinish() {

		PlayConfigManager.PlayConfigManagerFactory(wizardPage.getPlayProjectVersion(), wizardPage.getPlayProjectApplicationID());
		
		createNewProject();

		PlayConfigManager.createPlayProjectFile();

		return true;
	}

	private IProject createNewProject() {

		final String projectName = wizardPage.getProjectName();

		PlayConfigManager.setWorkspacePath(wizardPage.getLocationPathName());
		PlayConfigManager.setProjectName(projectName);

		projectHandle = PlayConfigManager.getProjectHandle(projectName);

        // get a project descriptor
        IPath defaultPath = Platform.getLocation();
        IPath newPath = wizardPage.getLocationPath();
        if(defaultPath.equals(newPath)){
            newPath = null;
        }else{
            //The user entered the path and it's the same as it'd be if he chose the default path.
            IPath withName = defaultPath.append(projectHandle.getName());
            if(newPath.toFile().equals(withName.toFile())){
                newPath = null;
            }
        }
        PlayConfigManager.setProjectPath(newPath == null ? projectName : "");

        IWorkspace workspace = ResourcesPlugin.getWorkspace();
        final IProjectDescription description = workspace.newProjectDescription(projectHandle.getName());
        description.setLocation(newPath);

        // define the operation to create a new project
        WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
            protected void execute(IProgressMonitor monitor) throws CoreException {
                createAndConfigProject(projectHandle, description,  monitor);
            }
        };

        // run the operation to create a new project
        try {
            getContainer().run(true, true, op);
        } catch (InterruptedException e) {
            return null;
        } catch (InvocationTargetException e) {
            Throwable t = e.getTargetException();
            if (t instanceof CoreException) {
                if (((CoreException) t).getStatus().getCode() == IResourceStatus.CASE_VARIANT_EXISTS) {
                    MessageDialog.openError(getShell(), "Unable to create project", "Another project with the same name (and different case) already exists.");
                } else {
                    ErrorDialog.openError(getShell(), "Unable to create project", null, ((CoreException) t).getStatus());
                }
            } else {
                MessageDialog.openError(getShell(), "Unable to create project", t.getMessage());
            }
            return null;
        }

        return projectHandle;
	}

	private void createAndConfigProject(IProject projectHandle, IProjectDescription description, IProgressMonitor monitor) throws CoreException {
        try {
            monitor.beginTask("", 2000);

            projectHandle.create(description, new SubProgressMonitor(monitor, 1000));

            if (monitor.isCanceled()){
                throw new OperationCanceledException();
            }
            projectHandle.open(IResource.BACKGROUND_REFRESH, new SubProgressMonitor(monitor, 1000));

            PlayConfigManager.createPlayProject(projectHandle, description, monitor);

        } catch (CoreException e) {
			e.printStackTrace();
		} finally {
            monitor.done();
        }
	}

}