package org.playframework.playtools.wizards;


import java.io.File;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.dialogs.IDialogPage;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Event;
import org.playframework.playtools.PlayToolsPlugin;
import org.playframework.playtools.preferences.PreferenceConstants;
import org.playframework.playtools.util.PlayToolsUtil;

/**
 * The "New" wizard page allows setting the container for the new file as well
 * as the file name. The page will only accept file name without the extension
 * OR with the extension that matches the expected one (mpe).
 */

public class PlayNewProjectWizardPage extends WizardPage {

    private static final int C_TEXT_WIDTH = 250;

    private IPath initialLocationTextValue;

    private Text projectNameText;

    private Button browseButton;

    private Label locationLabel;

    private Text locationPathText;

	private Combo playVersionCombo;

	private Combo applicationCombo;

    private boolean useDefaults = true;

    private String customLocationTextValue;

    private ISelection selection;


    private Listener projectNameModifyListener = new Listener() {
        public void handleEvent(Event event) {
            validate();
        }
    };

    private Listener projectLocationModifyListener = new Listener() {
        public void handleEvent(Event event) {
            validate();
        }
    };

    /**
     * Constructor for SampleNewWizardPage.
     *
     * @param pageName
     */
    public PlayNewProjectWizardPage(ISelection selection) {
        super("wizardPage");
        setTitle("Play Framework Project");
        setDescription("Create a new Play Framework Project.");
        this.selection = selection;
        initialLocationTextValue = Platform.getLocation();
    }

    /**
     * @see IDialogPage#createControl(Composite)
     */
    public void createControl(Composite parent) {
        Composite composite = new Composite(parent, SWT.NULL);
        composite.setLayout(new GridLayout());
        composite.setLayoutData(new GridData(GridData.FILL_BOTH));
        composite.setFont(parent.getFont());

        createProjectNameGroup(composite);
        createProjectLocationGroup(composite);
        createPlayVersionGroup(composite);

        initialize();

        validate();
        setControl(composite);
    }

    private final void createProjectNameGroup(Composite parent) {
        Font font = parent.getFont();

        Composite projectGroup = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();

        layout.numColumns = 2;
        projectGroup.setLayout(layout);
        projectGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        Label projectLabel = new Label(projectGroup, SWT.NONE);
        projectLabel.setFont(font);
        projectLabel.setText("&Project name:");

        projectNameText = new Text(projectGroup, SWT.BORDER);
        GridData data = new GridData(GridData.FILL_HORIZONTAL);
        data.widthHint = C_TEXT_WIDTH;
        projectNameText.setLayoutData(data);
        projectNameText.setFont(font);
        projectNameText.addListener(SWT.Modify, projectNameModifyListener);

    }

    private final void createProjectLocationGroup(Composite parent) {
        Font font = parent.getFont();

        Composite projectGroup = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        layout.numColumns = 3;

        projectGroup.setLayout(layout);
        projectGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        projectGroup.setFont(font);

        Label projectLocationLabel = new Label(projectGroup, SWT.NONE);
        projectLocationLabel.setFont(font);
        projectLocationLabel.setText("Project contents:");

        GridData labelData = new GridData();
        labelData.horizontalSpan = 3;
        projectLocationLabel.setLayoutData(labelData);

        final Button useDefaultsButton = new Button(projectGroup, SWT.CHECK | SWT.RIGHT);
        useDefaultsButton.setText("Use &default");
        useDefaultsButton.setSelection(useDefaults);
        useDefaultsButton.setFont(font);

        GridData buttonData = new GridData();
        buttonData.horizontalSpan = 3;
        useDefaultsButton.setLayoutData(buttonData);

        createUserSpecifiedProjectLocationGroup(projectGroup, !useDefaults);

        SelectionListener listener = new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                useDefaults = useDefaultsButton.getSelection();
                browseButton.setEnabled(!useDefaults);
                locationPathText.setEnabled(!useDefaults);
                locationLabel.setEnabled(!useDefaults);
                if (useDefaults) {
                    customLocationTextValue = locationPathText.getText();
                } else {
                    locationPathText.setText(customLocationTextValue);
                }
            }
        };
        useDefaultsButton.addSelectionListener(listener);
    }

    private void createUserSpecifiedProjectLocationGroup(Composite projectGroup, boolean enabled) {
        Font font = projectGroup.getFont();

        locationLabel = new Label(projectGroup, SWT.NONE);
        locationLabel.setFont(font);
        locationLabel.setText("Director&y");
        locationLabel.setEnabled(enabled);

        locationPathText = new Text(projectGroup, SWT.BORDER);
        GridData data = new GridData(GridData.FILL_HORIZONTAL);
        data.widthHint = C_TEXT_WIDTH;
        locationPathText.setLayoutData(data);
        locationPathText.setFont(font);
        locationPathText.setEnabled(enabled);

        browseButton = new Button(projectGroup, SWT.PUSH);
        browseButton.setFont(font);
        browseButton.setText("B&rowse");
        browseButton.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent event) {
                handleBrowse();
            }
        });

        browseButton.setEnabled(enabled);

        locationPathText.addListener(SWT.Modify, projectLocationModifyListener);
    }

    private final void createPlayVersionGroup(Composite parent) {
		Font font = parent.getFont();

		Composite projectGroup = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();

		layout.numColumns = 2;
		projectGroup.setLayout(layout);
		projectGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Label projectLabel = new Label(projectGroup, SWT.NONE);
		projectLabel.setFont(font);
		projectLabel.setText("Play Framework Version:");

		playVersionCombo = new Combo(projectGroup, SWT.READ_ONLY);

		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = C_TEXT_WIDTH;
		playVersionCombo.setLayoutData(data);
		playVersionCombo.setFont(font);

		Point point = playVersionCombo.getSize();
		point.x = C_TEXT_WIDTH;
		playVersionCombo.setSize(point);

		String location = PlayToolsPlugin.getDefault()
				.getPreferenceStore().getString(PreferenceConstants.P_PATH_PLAY);
		
		String items[] = PlayToolsUtil.paserLocationVersion(location);
		
		playVersionCombo.setItems(items);

		playVersionCombo.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				applicationCombo.setEnabled(getPlayProjectVersionID() == PlayToolsUtil.PLAY_VERSION_2);
			}
		});

		createApplicationGroup(projectGroup);
    }

    private final void createApplicationGroup(Composite projectGroup) {
		Font font = projectGroup.getFont();

		Label applicationLabel = new Label(projectGroup, SWT.NONE);
		applicationLabel.setFont(font);
		applicationLabel.setText("application:");

		applicationCombo = new Combo(projectGroup, SWT.READ_ONLY);

		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = C_TEXT_WIDTH;
		applicationCombo.setLayoutData(data);
		applicationCombo.setFont(font);

		Point point = applicationCombo.getSize();
		point.x = C_TEXT_WIDTH;
		applicationCombo.setSize(point);

		applicationCombo.setItems(PlayToolsUtil.application_items);
	}

    /**
     * Tests if the current workbench selection is a suitable container to use.
     */
    private void initialize() {
        projectNameText.setText("");

        if (initialLocationTextValue != null) {
            locationPathText.setText(initialLocationTextValue.toOSString());
        }
		playVersionCombo.select(0);
		applicationCombo.select(PlayToolsUtil.PLAY_PROJECT_APPLICATION_JAVA);
		applicationCombo.setEnabled(getPlayProjectVersionID() == PlayToolsUtil.PLAY_VERSION_2);
    }

    private void handleBrowse() {
        DirectoryDialog dialog = new DirectoryDialog(locationPathText.getShell());
        dialog.setMessage("Select the project contents directory.");

        String dirName = getLocationPathName();
        if (!dirName.equals("")) {
            File path = new File(dirName);
            if (path.exists()) {
                dialog.setFilterPath(new Path(dirName).toOSString());
            }
        }

        String selectedDirectory = dialog.open();
        if (selectedDirectory != null) {
            customLocationTextValue = selectedDirectory;
            locationPathText.setText(customLocationTextValue);
        }
    }

    /**
     * Ensures that both text fields are set.
     */
    private void validate() {

        String vertion = playVersionCombo.getText();
        if ("".equals(vertion)) {
            updateStatus("Play Framework installation directory not done");
            return;
        }

        String projectFieldContents = getProjectName();
        if ("".equals(projectFieldContents)) {
            updateStatus("Project name is empty");
            return;
        }

        IWorkspace workspace = ResourcesPlugin.getWorkspace();

        IStatus nameStatus = workspace.validateName(projectFieldContents, IResource.PROJECT);
        if (!nameStatus.isOK()) {
            updateStatus(nameStatus.getMessage());
            return;
        }

        String locationFieldContents = getLocationPathName();

        if ("".equals(locationFieldContents)) {
            updateStatus("Project location is empty");
            return;
        }

        IPath path = new Path("");
        if (!path.isValidPath(locationFieldContents)) {
            updateStatus("Project location is not valid");
            return;
        }


        IProject projectHandle = getProjectHandle();
        if (projectHandle.exists()) {
            setErrorMessage("Project already exists");
            return;
        }

        if (isDotProjectFileInLocation()) {
            updateStatus(".project found in: " + getLocationPath().toOSString() +" (use import project).");
            return;
        }
        updateStatus(null);
    }

    private boolean isDotProjectFileInLocation() {
        IPath path = getLocationPath();
        path = path.append(IProjectDescription.DESCRIPTION_FILE_NAME);
        return path.toFile().exists();
    }

    public IPath getLocationPath() {
        if (useDefaults)
            return initialLocationTextValue;

        return new Path(getLocationPathName());
    }

    private void updateStatus(String message) {
        setErrorMessage(message);
        setPageComplete(message == null);
    }

    public String getProjectName() {
        if (projectNameText == null) {
            return "";
        }
        return projectNameText.getText();
    }

    public String getLocationPathName() {
        if (locationPathText == null) {
            return "";
        }
        return locationPathText.getText().trim();
    }

    public IProject getProjectHandle() {
        return PlayToolsUtil.getProjectHandle(getProjectName());
    }

    public String getPlayProjectVersion() {
    	return playVersionCombo.getText();
    }
    
    public int getPlayProjectVersionID() {
    	return PlayToolsUtil.getPlayFrameworkVersionID(getPlayProjectVersion());
    }

    public String getPlayProjectApplication() {
    	if (getPlayProjectVersionID() == PlayToolsUtil.PLAY_VERSION_2) {
    		return applicationCombo.getText();
    	}
    	return "";
    }

    public int getPlayProjectApplicationID() {
    	if (getPlayProjectVersionID() == PlayToolsUtil.PLAY_VERSION_2) {
    		return applicationCombo.getSelectionIndex();
    	}
    	return PlayToolsUtil.PLAY_PROJECT_APPLICATION_JAVA;
    }
}